@@ -55,12 +55,15 @@ blavaan <- function(...,  # default lavaan arguments
         }
     }
 
-    # multilevel functionality not available
-    if("cluster" %in% dotNames) stop("blavaan ERROR: two-level models are not yet available.")
+    # multilevel functionality
+    if("cluster" %in% dotNames) {
+        cat("blavaan NOTE: two-level models are new, please report bugs!\nhttps://github.com/ecmerkle/blavaan/issues\n\n")
+        if(!(target == "stan")) stop("blavaan ERROR: two-level functionality is not available for ", target, ".")
+    }
   
-    # prior sampling only for stan
+    # prior predictives only for stan
     if(prisamp) {
-      if(target != 'stan') stop("blavaan ERROR: prior sampling currently only work for target='stan'.")
+      if(target != 'stan') stop("blavaan ERROR: prior predictives currently only work for target='stan'.")
       if(!('test' %in% dotNames)) {
         dotdotdot$test <- 'none'
         dotNames <- c(dotNames, "test")
@@ -210,7 +213,7 @@ blavaan <- function(...,  # default lavaan arguments
     dotdotdot$do.fit <- FALSE
     dotdotdot$se <- "none"; dotdotdot$test <- "none"
     # run for 1 iteration to obtain info about equality constraints, for npar
-    dotdotdot$control <- list(iter.max = 1); dotdotdot$warn <- TRUE
+    dotdotdot$control <- list(iter.max = 1, eval.max = 1); dotdotdot$warn <- TRUE
     dotdotdot$optim.force.converged <- TRUE
     if(target != "stan") dotdotdot$meanstructure <- TRUE
     dotdotdot$missing <- "direct"   # direct/ml creates error? (bug in lavaan?)
@@ -299,8 +302,12 @@ blavaan <- function(...,  # default lavaan arguments
     }
 
 
-    if("sample.mean" %in% names(dotdotdot) && !("data" %in% names(dotdotdot))) stop('blavaan ERROR: sample.mean cannot currently be used in place of data')
-
+    if("sample.cov" %in% names(dotdotdot)) {
+        if(!("data" %in% names(dotdotdot)) && target != "stan") stop('blavaan ERROR: sample.cov can only be used for target = "stan"')
+        if("cluster" %in% names(dotdotdot)) stop('blavaan ERROR: sample.cov cannot be used for two-level models')
+    }
+    if("meanstructure" %in% names(dotdotdot) && "sample.cov" %in% names(dotdotdot)) stop('blavaan ERROR: meanstructure is not currently allowed when sample.cov is supplied')
+  
     # call lavaan
     mcdebug <- FALSE
     if("debug" %in% dotNames){
@@ -316,12 +323,12 @@ blavaan <- function(...,  # default lavaan arguments
         if(!any(is.na(unlist(lavInspect(LAV, 'data'))))) dotdotdot$missing <- "listwise"
     }
 
-    # for initial values/parameter setup:
-    if(jag.do.fit) {
+    # for initial values/some parameter setup:
+    if(jag.do.fit){
         LAV2 <- try(do.call("lavaan", dotdotdot), silent = TRUE)
         if(!inherits(LAV2, 'try-error')) LAV <- LAV2
     }
-        
+
     if(LAV@Data@data.type == "moment") {
         if(target != "stan") stop('blavaan ERROR: full data are required for ', target, ' target.\n  Try target="stan", or consider using kd() from package semTools.')
     }
@@ -468,9 +475,6 @@ blavaan <- function(...,  # default lavaan arguments
     lavoptions$estimator <- "Bayes"
     lavoptions$se        <- "standard"
     lavoptions$test <- "standard"
-    if(ordmod) {
-        cat("blavaan NOTE: ordinal models are new, please report bugs!\nhttps://github.com/ecmerkle/blavaan/issues\n\n")
-    }
     if("test" %in% dotNames) {
         if(dotdotdot$test == "none") lavoptions$test <- "none"
     } else {
@@ -481,12 +485,14 @@ blavaan <- function(...,  # default lavaan arguments
     }
     if(!jag.do.fit){
       lavoptions$test <- "none"
+      lavoptions$se <- "none"
     }
     lavoptions$missing   <- "ml"
     lavoptions$cp        <- cp
     lavoptions$dp        <- dp
     lavoptions$prisamp   <- prisamp
     lavoptions$target    <- target
+    lavoptions$optim.method <- "mcmc"
     if("llnsamp" %in% names(mcmcextra$data)){
         if(length(mcmcextra$data$llnsamp) > 1 ||
            (!inherits(mcmcextra$data$llnsamp, "numeric") &&
@@ -537,45 +543,65 @@ blavaan <- function(...,  # default lavaan arguments
                            silent = TRUE)
 
                 if(!inherits(l2s, "try-error")){
-                    lavpartable$prior[as.numeric(rownames(l2s$lavpartable))] <- l2s$lavpartable$prior
-                    ldargs <- c(l2s$dat, list(lavpartable = l2s$lavpartable, dumlv = l2s$dumlv,
-                                              save_lvs = save.lvs, do_test = !(lavoptions$test == "none") ))
+                    l2slev2 <- try(lav2stanmarg(lavobject = LAV, dp = dp,
+                                                n.chains = n.chains, mcmcextra = mcmcextra,
+                                                inits = initsin, wiggle = wiggle,
+                                                wiggle.sd = wiggle.sd, prisamp = prisamp,
+                                                level = 2L, indat = l2s$dat))
+                
+                    if(!inherits(l2slev2, "try-error")){
+                        l2s$dat <- c(l2s$dat, l2slev2$dat)
+                        l2s$dat <- l2s$dat[!duplicated(names(l2s$dat))]
 
-                    ## add priors to lavpartable, including wiggle
-                    if(length(wiggle) > 0){
-                      wigrows <- which(l2s$wigpris != "")
-                      lavpartable$prior[as.numeric(rownames(l2s$lavpartable))[wigrows]] <- l2s$wigpris[wigrows]
-                    }
+                        l2s$free2 <- c(l2s$free2, l2slev2$free2)
 
-                    jagtrans <- try(do.call("stanmarg_data", ldargs), silent = TRUE)
+                        l2s$lavpartable <- rbind(l2s$lavpartable, l2slev2$lavpartable)
+                        l2s$wigpris <- c(l2s$wigpris, l2slev2$wigpris)
+                        l2s$init <- lapply(1:length(l2s$init), function(i) c(l2s$init[[i]], l2slev2$init[[i]]))
+                  
+                        lavpartable$prior[as.numeric(rownames(l2s$lavpartable))] <- l2s$lavpartable$prior
+                        ldargs <- c(l2s$dat, list(lavpartable = l2s$lavpartable, dumlv = l2s$dumlv,
+                                                  dumlv_c = l2slev2$dumlv,
+                                                  save_lvs = save.lvs, do_test = !(lavoptions$test == "none") ))
 
-                    if(inherits(jagtrans, "try-error")) stop(jagtrans)
+                        ## add priors to lavpartable, including wiggle
+                        if(length(wiggle) > 0){
+                            wigrows <- which(l2s$wigpris != "")
+                            lavpartable$prior[as.numeric(rownames(l2s$lavpartable))[wigrows]] <- l2s$wigpris[wigrows]
+                        }
 
-                    ## add lkj for unrestricted psi
-                    if(jagtrans$fullpsi == 1L){
-                      psirows <- which(l2s$lavpartable$mat == "lvrho")
-                      lavpartable$prior[as.numeric(rownames(l2s$lavpartable))[psirows]] <- paste0("lkj_corr(", jagtrans$psi_r_alpha[1], ")")
-                    }
+                        jagtrans <- try(do.call("stanmarg_data", ldargs), silent = TRUE)
+                        if(inherits(jagtrans, "try-error")) stop(jagtrans)
+                        
+                        ## add lkj for unrestricted psi
+                        if(jagtrans$fullpsi == 1L){
+                            psirows <- which(l2s$lavpartable$mat == "lvrho")
+                            lavpartable$prior[as.numeric(rownames(l2s$lavpartable))[psirows]] <- paste0("lkj_corr(", jagtrans$psi_r_alpha[1], ")")
+                        }
 
-                    jagtrans <- list(data = jagtrans,
-                                     monitors = c("ly_sign",
-                                           #"lx_sign",
-                                           "bet_sign", "g_sign",
-                                           "Theta_cov", "Theta_var",
-                                           #"Theta_x_cov", "Theta_x_var",
-                                           "Psi_cov", "Psi_var",
-                                           #"Ph_cov", "Ph_var",
-                                           "Nu_free", "Alpha_free", "Tau_free"))
-                    if(lavoptions$test != "none"){
-                      jagtrans$monitors <- c(jagtrans$monitors, "log_lik", "log_lik_sat", "ppp")
-                    }
+                        stanmon <- c("ly_sign", "bet_sign", "Theta_cov", "Theta_var",
+                                     "Psi_cov", "Psi_var", "Nu_free", "Alpha_free", "Tau_free")
+                        if(lavoptions$.multilevel){
+                          stanmon <- c(stanmon, paste0(stanmon, "_c"))
+                          stanmon <- stanmon[-which(stanmon == "Tau_free_c")]
+                        }
+                        if(lavoptions$test != "none"){
+                          stanmon <- c(stanmon, c("log_lik", "log_lik_sat", "ppp"))
+                        } else if(!lavInspect(LAV, 'categorical') && (lavoptions$.multilevel || lavInspect(LAV, 'meanstructure'))){
+                          stanmon <- c(stanmon, "log_lik")
+                        }
 
-                    if("init" %in% names(l2s)){
-                      jagtrans <- c(jagtrans, list(inits = l2s$init))
-                    }
+                        jagtrans <- list(data = jagtrans, monitors = stanmon)
 
-                    if(ordmod && save.lvs){
-                      jagtrans$monitors <- c(jagtrans$monitors, "YXostar")
+                        if("init" %in% names(l2s)){
+                            jagtrans <- c(jagtrans, list(inits = l2s$init))
+                        }
+
+                        if(ordmod && save.lvs){
+                            jagtrans$monitors <- c(jagtrans$monitors, "YXostar")
+                        }
+                    } else {
+                        jagtrans <- l2slev2
                     }
                 } else {
                     jagtrans <- l2s
@@ -590,15 +616,16 @@ blavaan <- function(...,  # default lavaan arguments
                     jagtrans$monitors <- c(jagtrans$monitors, mcmcextra$monitor)
                 }
                 if("data" %in% names(mcmcextra) & "moment_match_k_threshold" %in% names(mcmcextra$data)){
-                    moment_match_monitors <- c(
-                      c("Lambda_y_free", "Gamma_free", "B_free", 
+                    ## FIXME these do not cover level 2 parameters
+                    moment_match_monitors <- c("Lambda_y_free", "B_free", 
                         "Theta_sd_free", "Theta_r_free", "Psi_sd_free", 
-                        "Psi_r_mat", "Psi_r_free", "Tau_ufree", 
-                        "z_aug", "ly_sign", "bet_sign", "g_sign",
-                        "Theta_cov", "Theta_var", "Psi_cov", "Psi_var", 
-                        "Nu_free", "Alpha_free", "Tau_free", "log_lik", 
-                        "log_lik_sat", "ppp")
-                    )
+                        "Psi_r_mat", "Psi_r_free", "Nu_free", "Alpha_free")
+                    moment_match_monitors <- c(moment_match_monitors,
+                                               paste0(moment_match_monitors, "_c"))
+                    moment_match_monitors <- c(moment_match_monitors, "Tau_ufree", 
+                                               "z_aug", "ly_sign", "bet_sign", "Theta_cov",
+                                               "Theta_var", "Psi_cov", "Psi_var", "Tau_free",
+                                               "log_lik", "log_lik_sat", "ppp")
                     jagtrans$monitors <- c(jagtrans$monitors, moment_match_monitors[!(moment_match_monitors %in% jagtrans$monitors)])
                 }
                 if("data" %in% names(mcmcextra)){
@@ -686,6 +713,7 @@ blavaan <- function(...,  # default lavaan arguments
                     rjcall <- "stan"
                 } else if(usevb){
                     rjcall <- "vb"
+                    rjarg$init <- rjarg$init[[1]]
                 } else if(target == "cmdstan"){
                     fname <- paste0("stanmarg_", packageDescription("blavaan")["Version"])
                     fdir <- paste0(cmdstanr::cmdstan_path(), "/")
@@ -740,7 +768,8 @@ blavaan <- function(...,  # default lavaan arguments
 
         timing$Estimate <- (proc.time()[3] - start.time)
         start.time <- proc.time()[3]
-
+        if(jag.do.fit) cat("Computing post-estimation metrics (including lvs if requested)...\n")
+        
         ## FIXME: there is no pars argument. this saves all parameters and uses unnecessary memory
         ## see res@sim and line 284 of stan_csv.R... might cut it down manually
         if(target == "cmdstan"){
@@ -755,7 +784,34 @@ blavaan <- function(...,  # default lavaan arguments
                                   res)
           stansumm <- parests$stansumm
         } else {
-          parests <- coeffun_stanmarg(lavpartable, lavInspect(LAV, 'free'), l2s$free2, jagtrans$data, res)
+          if(jag.do.fit) {
+            draw_mat <- as.matrix(res)
+          } else {
+            draw_mat <- NULL
+          }
+          
+          if(lavoptions$.multilevel) {
+            Ng <- lavInspect(LAV, 'ngroups')
+            parests <- coeffun_stanmarg(lavpartable, lavInspect(LAV, 'free')[2*(1:Ng) - 1], l2s$free2, jagtrans$data, res, colnames(draw_mat))
+            parests2 <- coeffun_stanmarg(lavpartable, lavInspect(LAV, 'free')[2*(1:Ng)], l2s$free2, jagtrans$data, res, colnames(draw_mat), level = 2L)
+            ## combine list elements of the partables
+            parests$lavpartable <- mapply("c", parests$lavpartable, parests2$lavpartable, SIMPLIFY = FALSE)
+            ##parests$vcorr <- ## need level 1 by level 2!
+            ## reorder to match original partable ordering
+            idord <- parests$lavpartable$id
+            freeid <- idord[parests$lavpartable$free > 0]
+            parests$lavpartable <- lapply(parests$lavpartable, function(x) x[order(idord)])
+            parests$x <- c(parests$x, parests2$x)
+            parests$sd <- c(parests$sd, parests2$sd)
+            parests$x <- parests$x[order(freeid)]
+            parests$sd <- parests$sd[order(freeid)]
+          } else {
+            parests <- coeffun_stanmarg(lavpartable, lavInspect(LAV, 'free'), l2s$free2, jagtrans$data, res, colnames(draw_mat))
+          }
+
+          if(jag.do.fit) {
+            parests$vcorr <- cor(draw_mat[, with(parests$lavpartable, stansumnum[free > 0]), drop=FALSE])
+          }
           stansumm <- parests$stansumm
         }
         x <- parests$x
@@ -763,6 +819,10 @@ blavaan <- function(...,  # default lavaan arguments
 
         if(jag.do.fit){
             lavmodel <- lav_model_set_parameters(lavmodel, x = x)
+            LAV@ParTable <- lavpartable
+            LAV@Model <- lavmodel
+            LAV@external$mcmcout <- res
+            
             if(target == "jags"){
                 attr(x, "iterations") <- res$sample
                 sample <- res$sample
@@ -777,11 +837,31 @@ blavaan <- function(...,  # default lavaan arguments
                 }
                 ## lvs now in R instead of Stan
                 if(save.lvs & target == "stan"){
-                    stanlvs <- samp_lvs(res, lavmodel, parests$lavpartable, jagtrans$data, lavInspect(LAV, "categorical"))
-                    if(dim(stanlvs)[3L] > 0){
-                        lvsumm <- as.matrix(rstan::monitor(stanlvs, print=FALSE))
-                        cmatch <- match(colnames(stansumm), colnames(lvsumm))
-                        stansumm <- rbind(stansumm, lvsumm[,cmatch])
+                    if(lavoptions$.multilevel){
+                        ## this handles dummy lvs so that we use the appropriate alpha:
+                        lav_eeta <- getFromNamespace("computeEETA", "lavaan")
+                        eeta <- lav_eeta(lavmodel = lavmodel, lavsamplestats = lavsamplestats)
+                        stanlvs <- samp_lvs_2lev(res, lavmodel, lavsamplestats, lavdata, parests$lavpartable, jagtrans$data, eeta)
+                    } else {
+                        stanlvs <- list(samp_lvs(res, lavmodel, parests$lavpartable, jagtrans$data, eeta = NULL, lavInspect(LAV, "categorical")))
+                    }
+
+                    for(j in 1:(1 + lavoptions$.multilevel)){
+                        if(dim(stanlvs[[j]])[3L] > 0){
+                            lvsumm <- as.matrix(rstan::monitor(stanlvs[[j]], print=FALSE))
+                            cmatch <- match(colnames(stansumm), colnames(lvsumm))
+                            stansumm <- rbind(stansumm, lvsumm[,cmatch])
+                        }
+                    }
+
+                    if(lavoptions$.multilevel){
+                        stanlvs <- simplify2array( sapply(1:nrow(stanlvs[[1]]), function(i) cbind(stanlvs[[1]][i,,],
+                                                                                                  stanlvs[[2]][i,,]), simplify = FALSE) )
+                        if(with(jagtrans$data, length(usepsi) + length(usepsi_c) > 0)){
+                            stanlvs <- aperm(stanlvs, c(3, 1, 2))
+                        }
+                    } else {
+                        stanlvs <- stanlvs[[1]]
                     }
                 }
                 # burnin + sample already defined, will be saved in
@@ -801,10 +881,6 @@ blavaan <- function(...,  # default lavaan arguments
         attr(x, "control") <- bcontrol
 
         ## log-likelihoods
-        LAV@ParTable <- lavpartable
-        LAV@Model <- lavmodel
-        LAV@external$mcmcout <- res
-        LAV@Options$se <- "standard"
         LAV@Options$target <- "jags" ## to ensure computation in R, vs extraction of the
                                      ## log-likehoods from Stan
         ## FIXME: modify so that fx is commensurate with logl from Stan
@@ -812,7 +888,7 @@ blavaan <- function(...,  # default lavaan arguments
         attr(x, "fx") <- get_ll(lavobject = LAV, standata = rjarg$data)[1]
         LAV@Options$target <- target
 
-        if(save.lvs && jag.do.fit && !ordmod && lavInspect(LAV, "meanstructure")) {
+        if(save.lvs && jag.do.fit && !ordmod && !lavoptions$.multilevel && lavInspect(LAV, "meanstructure")) {
             if(target == "jags"){
                 fullpmeans <- summary(make_mcmc(res))[[1]][,"Mean"]
             } else {
@@ -844,7 +920,6 @@ blavaan <- function(...,  # default lavaan arguments
     ## fx is mean ll, where ll is marginal log-likelihood (integrate out lvs)
     casells <- NULL
     if(lavoptions$test != "none") {
-      cat("Computing posterior predictives...\n")
       lavmcmc <- make_mcmc(res)
       LAV@Options <- lavoptions
 
@@ -865,7 +940,7 @@ blavaan <- function(...,  # default lavaan arguments
         sampkls <- NA
       }
       
-      if(save.lvs && !ordmod && lavInspect(LAV, "meanstructure")) {
+      if(save.lvs && !ordmod && !lavoptions$.multilevel && lavInspect(LAV, "meanstructure")) {
         if(target == "stan"){
           lavmcmc <- make_mcmc(res, stanlvs) ## add on lvs
         }
@@ -944,6 +1019,7 @@ blavaan <- function(...,  # default lavaan arguments
                              x           = x,
                              VCOV        = VCOV,
                              TEST        = TEST)
+
     ## add SE and SD-Bayes factor to lavpartable
     ## (code around line 270 of blav_object_methods
     ##  can be removed with this addition near line 923
@@ -986,7 +1062,7 @@ blavaan <- function(...,  # default lavaan arguments
       if(save.lvs & target=="stan") extslot <- c(extslot, list(stanlvs = stanlvs))
     }
     if(jags.ic) extslot <- c(extslot, list(sampkls = sampkls))
-    if(save.lvs && !ordmod && lavInspect(LAV, "meanstructure")) {
+    if(save.lvs && !ordmod && !lavoptions$.multilevel && lavInspect(LAV, "meanstructure")) {
       extslot <- c(extslot, list(cfx = cfx, csamplls = csamplls))
       if(jags.ic) extslot <- c(extslot, list(csampkls = csampkls))
     }
@@ -999,6 +1075,7 @@ blavaan <- function(...,  # default lavaan arguments
     
     # 10. construct blavaan object
     blavaan <- new("blavaan",
+                   version      = as.character( packageVersion('blavaan') ),
                    call         = mc,                  # match.call
                    timing       = timing,              # list
                    Options      = lavoptions,          # list
@@ -1061,7 +1138,7 @@ bcfa <- bsem <- function(..., cp = "srs", dp = NULL,
     mc$auto.cov.y      = TRUE
     mc$auto.th         = TRUE
     mc$auto.delta      = TRUE
-    mc[[1L]] <- quote(blavaan::blavaan)
+    mc[[1L]] <- quote(blavaan)
 
     ## change defaults depending on jags vs stan
     sampargs <- c("burnin", "sample", "adapt")
@@ -1104,7 +1181,7 @@ bgrowth <- function(..., cp = "srs", dp = NULL,
     mc$auto.cov.y      = TRUE
     mc$auto.th         = TRUE
     mc$auto.delta      = TRUE
-    mc[[1L]] <- quote(blavaan::blavaan)
+    mc[[1L]] <- quote(blavaan)
 
     ## change defaults depending on jags vs stan
     sampargs <- c("burnin", "sample", "adapt")
